@@ -6,7 +6,7 @@
  Description: formatted call C interface (extension module)
  License:
 
-   Copyright (c) 2007-2015 Daniel Adler <dadler@uni-goettingen.de>, 
+   Copyright (c) 2007-2018 Daniel Adler <dadler@uni-goettingen.de>, 
                            Tassilo Philipp <tphilipp@potion-studios.com>
 
    Permission to use, copy, modify, and distribute this software for any
@@ -28,13 +28,14 @@
 #include "dyncall_callf.h"
 
 
-// Shareable implementation for argument binding used in ArgF and CallF  below.
+/* Shareable implementation for argument binding used in ArgF and CallF below. */
 static void dcArgF_impl(DCCallVM* vm, const DCsigchar** sigptr, va_list args)
 {
   DCsigchar ch;
   dcReset(vm);
   while((ch=*(*sigptr)++) != '\0' && ch != DC_SIGCHAR_ENDARG) {
     switch(ch) {
+      /* types */
       case DC_SIGCHAR_BOOL:      dcArgBool    (vm, (DCbool)           va_arg(args, DCint     )); break;
       case DC_SIGCHAR_CHAR:      dcArgChar    (vm, (DCchar)           va_arg(args, DCint     )); break;
       case DC_SIGCHAR_UCHAR:     dcArgChar    (vm, (DCchar)(DCuchar)  va_arg(args, DCint     )); break;
@@ -50,6 +51,14 @@ static void dcArgF_impl(DCCallVM* vm, const DCsigchar** sigptr, va_list args)
       case DC_SIGCHAR_DOUBLE:    dcArgDouble  (vm, (DCdouble)         va_arg(args, DCdouble  )); break;
       case DC_SIGCHAR_POINTER:   dcArgPointer (vm, (DCpointer)        va_arg(args, DCpointer )); break;
       case DC_SIGCHAR_STRING:    dcArgPointer (vm, (DCpointer)        va_arg(args, DCpointer )); break;
+      /* calling convention modes */
+      case DC_SIGCHAR_CC_PREFIX:
+        if(*((*sigptr)+1) != '\0') {
+          DCint mode = dcGetModeFromCCSigChar(*(*sigptr)++);
+          if(mode != DC_ERROR_UNSUPPORTED_MODE)
+            dcMode(vm, mode);
+        }
+        break;
     }
   }
 }
@@ -73,22 +82,22 @@ void dcVCallF(DCCallVM* vm, DCValue* result, DCpointer funcptr, const DCsigchar*
   dcArgF_impl(vm, &ptr, args);
 
   switch(*ptr) {
-    case DC_SIGCHAR_VOID:                   dcCallVoid         (vm,funcptr); break;
-    case DC_SIGCHAR_BOOL:       result->B = dcCallBool         (vm,funcptr); break;
-    case DC_SIGCHAR_CHAR:       result->c = dcCallChar         (vm,funcptr); break;
-    case DC_SIGCHAR_UCHAR:      result->C = (DCuchar)dcCallChar(vm,funcptr); break;
-    case DC_SIGCHAR_SHORT:      result->s = dcCallShort        (vm,funcptr); break;
-    case DC_SIGCHAR_USHORT:     result->S = dcCallShort        (vm,funcptr); break;
-    case DC_SIGCHAR_INT:        result->i = dcCallInt          (vm,funcptr); break;
-    case DC_SIGCHAR_UINT:       result->I = dcCallInt          (vm,funcptr); break;
-    case DC_SIGCHAR_LONG:       result->j = dcCallLong         (vm,funcptr); break;
-    case DC_SIGCHAR_ULONG:      result->J = dcCallLong         (vm,funcptr); break;
-    case DC_SIGCHAR_LONGLONG:   result->l = dcCallLongLong     (vm,funcptr); break;
-    case DC_SIGCHAR_ULONGLONG:  result->L = dcCallLongLong     (vm,funcptr); break;
-    case DC_SIGCHAR_FLOAT:      result->f = dcCallFloat        (vm,funcptr); break;
-    case DC_SIGCHAR_DOUBLE:     result->d = dcCallDouble       (vm,funcptr); break;
-    case DC_SIGCHAR_POINTER:    result->p = dcCallPointer      (vm,funcptr); break;
-    case DC_SIGCHAR_STRING:     result->Z = dcCallPointer      (vm,funcptr); break;
+    case DC_SIGCHAR_VOID:                   dcCallVoid             (vm,funcptr); break;
+    case DC_SIGCHAR_BOOL:       result->B = dcCallBool             (vm,funcptr); break;
+    case DC_SIGCHAR_CHAR:       result->c = dcCallChar             (vm,funcptr); break;
+    case DC_SIGCHAR_UCHAR:      result->C = (DCuchar)dcCallChar    (vm,funcptr); break;
+    case DC_SIGCHAR_SHORT:      result->s = dcCallShort            (vm,funcptr); break;
+    case DC_SIGCHAR_USHORT:     result->S = dcCallShort            (vm,funcptr); break;
+    case DC_SIGCHAR_INT:        result->i = dcCallInt              (vm,funcptr); break;
+    case DC_SIGCHAR_UINT:       result->I = dcCallInt              (vm,funcptr); break;
+    case DC_SIGCHAR_LONG:       result->j = dcCallLong             (vm,funcptr); break;
+    case DC_SIGCHAR_ULONG:      result->J = dcCallLong             (vm,funcptr); break;
+    case DC_SIGCHAR_LONGLONG:   result->l = dcCallLongLong         (vm,funcptr); break;
+    case DC_SIGCHAR_ULONGLONG:  result->L = dcCallLongLong         (vm,funcptr); break;
+    case DC_SIGCHAR_FLOAT:      result->f = dcCallFloat            (vm,funcptr); break;
+    case DC_SIGCHAR_DOUBLE:     result->d = dcCallDouble           (vm,funcptr); break;
+    case DC_SIGCHAR_POINTER:    result->p = dcCallPointer          (vm,funcptr); break;
+    case DC_SIGCHAR_STRING:     result->Z = (DCstring)dcCallPointer(vm,funcptr); break;
   }
 }
 
